from django.contrib import admin, messages

from .forms import EditIpAddressForm
from .models import EditIpAddress


class EditIpAddressAdmin(admin.ModelAdmin):
    raw_id_fields = ("user",)
    list_select_related = ("user",)
    list_display = ("ips", "user")
    ordering = ("user__email",)
    search_fields = ("ips", "user__email")

    form = EditIpAddressForm


admin.site.register(EditIpAddress, EditIpAddressAdmin)



import pytest

from django_ip_access.middleware import IpAccessMiddleware

pytestmark = pytest.mark.django_db


class TestIpAccessMiddleware:
    @staticmethod
    def test_init():
        # GIVEN / WHEN
        ip_access_middleware = IpAccessMiddleware("dump response")

        # THEN
        assert ip_access_middleware.get_response == "dump response"

    def test_anonymous_user_without_existing_ip_address(self, request_builder):
        request = request_builder.get
        ip_access_middleware = IpAccessMiddleware(request)

        ip_access_middleware(request())

        assert ip_access_middleware.get_response().path == "/"

    def test_anonymous_user_with_existing_ip_address(self, ip, mocker, request_builder):
        # GIVEN
        request = request_builder.get
        ip_access_middleware = IpAccessMiddleware(request)

        # WHEN
        mocker.patch(
            "django_ip_access.middleware.get_client_ip", return_value=(ip.ip, False)
        )
        ip_access_middleware(request())

        # THEN
        assert ip_access_middleware.get_response().path == "/"

    def test_authenticated_user(self, user, request_builder):
        # GIVEN
        request = request_builder.get
        ip_access_middleware = IpAccessMiddleware(request)

        # WHEN
        ip_access_middleware(request(user=user))

        # THEN
        assert ip_access_middleware.get_response().path == "/"



import ipaddress

from django.contrib.auth import get_user_model
from django.db import models

User = get_user_model()


class EditIpAddress(models.Model):
    ips = models.TextField()
    user = models.OneToOneField(User, on_delete=models.CASCADE)

    class Meta:
        verbose_name = "Edit Authorized IP address"
        verbose_name_plural = "Edit Authorized IP addresses"

    def __str__(self):
        return str(self.user)

    def _generate_ips_list(self):
        ips_list = []
        for ip in self.ips.split():
            try:
                ips_list.append(str(ipaddress.ip_address(ip)))
            except ValueError:
                ips_list += [str(el) for el in ipaddress.ip_network(ip).hosts()]

        return ips_list

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)

        self.ipaddress_set.all().delete()

        ips_list = self._generate_ips_list()

        ip_addresses_to_create = []
        for ip in ips_list:
            ip_addresses_to_create.append(
                IpAddress(user=self.user, ip=ip, edit_ip_address=self)
            )

        IpAddress.objects.bulk_create(ip_addresses_to_create)


class IpAddress(models.Model):
    ip = models.GenericIPAddressField(unique=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    edit_ip_address = models.ForeignKey(
        "EditIpAddress", on_delete=models.CASCADE, null=True
    )

    class Meta:
        verbose_name = "Authorized IP address"
        verbose_name_plural = "Authorized IP addresses"

    def __str__(self):
        return self.ip



# Generated by Django 2.2.9 on 2020-02-04 10:24

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="IpAddress",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("ip", models.GenericIPAddressField(unique=True)),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Authorized IP address",
                "verbose_name_plural": "Authorized IP addresses",
            },
        )
    ]



from django.conf import settings
from django.contrib.auth import authenticate, login
from django.core.cache import cache
from ipware import get_client_ip


class IpAccessMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if request.user and request.user.is_authenticated:
            return self.get_response(request)

        ip, _ = get_client_ip(request)
        cache_key = f"{settings.IP_ACCESS_CACHE_KEY_PREFIX}_{ip}"

        if not cache.get(cache_key):
            user = authenticate(request, ip=ip)
            cache.set(cache_key, True, settings.IP_ACCESS_CACHE_TTL)

            if user:
                login(
                    request, user, backend="django_ip_access.backends.IpAccessBackend"
                )

        return self.get_response(request)



# Generated by Django 2.2.11 on 2020-03-20 10:48

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("django_ip_access", "0002_auto_20200204_1233"),
    ]

    operations = [
        migrations.CreateModel(
            name="EditIpAddress",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("ips", models.TextField()),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Edit Authorized IP address",
                "verbose_name_plural": "Edit Authorized IP addresses",
            },
        ),
        migrations.AddField(
            model_name="ipaddress",
            name="edit_ip_address",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="django_ip_access.EditIpAddress",
            ),
        ),
    ]



import pytest

from django_ip_access.backends import IpAccessBackend

pytestmark = pytest.mark.django_db


class TestIpAccessBackend:
    @staticmethod
    def test_get_user_user_exists(user):
        user = IpAccessBackend.get_user(user.id)
        assert user

    @staticmethod
    def test_get_user_without_existing_user(user):
        user = IpAccessBackend.get_user(user.id + 1)
        assert not user

    @staticmethod
    def test_authenticate_with_inactive_user(request_builder, user):
        user.is_active = False
        user = IpAccessBackend.authenticate(request_builder.get(user))

        assert not user

    @staticmethod
    def test_authenticate_with_active_user(request_builder, ip):
        user = IpAccessBackend.authenticate(request_builder.get(ip.user), ip=ip.ip)

        assert user



# Generated by Django 2.2.9 on 2020-02-04 11:33

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("django_ip_access", "0001_initial")]

    operations = [
        migrations.AlterField(
            model_name="ipaddress",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
            ),
        )
    ]



import pytest

from django_ip_access.forms import EditIpAddressForm
from django_ip_access.models import IpAddress

pytestmark = pytest.mark.django_db


class TestEditIpAddressForm:
    def test_form_works(self, user):
        # GIVEN
        ips = ["127.0.0.1", "127.0.0.2", "128.0.0.0/24"]
        data = {"ips": "\n".join(ips), "user": user.pk}

        # WHEN
        form = EditIpAddressForm(data=data)

        # THEN
        assert form.is_valid()
        form.save()
        assert IpAddress.objects.all().count() == 256

    def test_form_with_wrong_ip_formatting(self, user):
        # GIVEN
        ips = ["127.0.0.1", "wrong.ip"]
        data = {"ips": "\n".join(ips), "user": user.pk}

        # WHEN
        form = EditIpAddressForm(data=data)

        # THEN
        assert not form.is_valid()
        assert "wrong.ip is not well-formatted." in form.errors["ips"]

    def test_remove_duplicates(self, user):
        # GIVEN
        ips = ["127.0.0.1", "127.0.0.1", "127.0.0.2"]
        data = {"ips": "\n".join(ips), "user": user.pk}

        # WHEN
        form = EditIpAddressForm(data=data)

        # THEN
        assert form.is_valid()
        form.save()
        assert IpAddress.objects.filter(ip__in=ips).count() == 2

    def test_ip_already_exists(self, user, ip):
        """Raise a validation error in case the ip exists for another user"""
        # GIVEN
        ips = [ip.ip]
        data = {"ips": "\n".join(ips), "user": user.pk}

        # WHEN
        form = EditIpAddressForm(data=data)

        # THEN
        assert not form.is_valid()
        assert f"{ip.ip} already exists for user {ip.user}." in form.errors["ips"]



from setuptools import setup

from os import path

from django_ip_access import __version__

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="django-ip-access",
    version=__version__,
    description="Access a Django app with authorized IP address",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/briefmnews/django-ip-access",
    author="Brief.me",
    author_email="tech@brief.me",
    license="GNU General Public License v3.0",
    packages=["django_ip_access", "django_ip_access.migrations"],
    python_requires=">=3.9",
    install_requires=["Django>=4.2", "django-ipware>=2.1"],
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 4.2",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    include_package_data=True,
    zip_safe=False,
)



from django.contrib.auth import get_user_model

from .models import IpAddress

User = get_user_model()


class IpAccessBackend:
    """Authentication with IP address"""

    @staticmethod
    def authenticate(request, ip=None):
        try:
            ip_address = IpAddress.objects.get(ip=ip, user__is_active=True)
        except IpAddress.DoesNotExist:
            return None

        return ip_address.user

    @staticmethod
    def get_user(user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



import ipaddress

from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from .models import EditIpAddress, IpAddress


class EditIpAddressForm(forms.ModelForm):
    class Meta:
        model = EditIpAddress
        help_texts = {"ips": _("One ip address per row.")}
        fields = "__all__"

    def clean_ips(self):
        """
        Cleans and validates the IPs field.
        - Ensures each IP is well-formatted.
        - Removes duplicates.
        - Validates against existing IPs in the database.
        """
        ips = self.cleaned_data.get("ips")

        list_ips = ips.split()

        # Validate formatting and collect unique IPs
        unique_ips = set()
        errors = []
        for ip in list_ips:
            try:
                ipaddress.ip_network(ip)  # Validate IP format
                unique_ips.add(ip)
            except ValueError:
                errors.append(_(f"{ip} is not well-formatted."))

        # Check for existing IPs in the database
        existing_ips = (
            IpAddress.objects.exclude(edit_ip_address=self.instance or None)
            .filter(ip__in=unique_ips)
            .select_related("user")
        )
        for ip_entry in existing_ips:
            errors.append(_(f"{ip_entry.ip} already exists for user {ip_entry.user}."))
            unique_ips.discard(ip_entry.ip)

        if errors:
            raise ValidationError(errors)

        return "\n".join(sorted(unique_ips))



import pytest
from faker import Faker

from django_ip_access.models import EditIpAddress, IpAddress

faker = Faker()
pytestmark = pytest.mark.django_db


class TestEditIpAddress:
    def test_str(self, edit_ip):
        assert edit_ip.__str__() == str(edit_ip.user)

    def test_save(self, edit_ip):
        # GIVEN
        user = edit_ip.user

        # WHEN
        ips_list = edit_ip._generate_ips_list()
        saved_ips = IpAddress.objects.filter(user=user).values_list("ip", flat=True)

        # THEN
        assert set(saved_ips) == set(ips_list)


class TestIpAddress:
    def test_str(self, ip):
        assert ip.__str__() == ip.ip

    def test_one_user_can_have_multiple_ip_access(self, ip):
        # GIVEN
        user = ip.user

        # WHEN
        second_ip = IpAddress(user=user, ip=faker.ipv4())
        second_ip.save()

        # THEN
        assert user.ipaddress_set.all().count() == 2
